@@ -1,7 +1,8 @@
 use std::convert::Infallible;
 use std::fmt::Formatter;
+use std::marker::PhantomData;
 use std::mem;
-use std::ops::{Deref, FromResidual};
+use std::ops::FromResidual;
 
 pub use protobuf::{CodedOutputStream, Message, ProtobufEnum};
 pub use protobuf::well_known_types::Any;
@@ -12,23 +13,16 @@ pub type Method = i32;
 pub type WasmMethod = Method;
 pub type VmMethod = Method;
 pub type CtxId = i32;
-pub type Result<Data> = std::result::Result<Data, CodeMsg>;
+pub type Result<T> = std::result::Result<T, CodeMsg>;
 
-#[derive(Debug, Copy, Clone)]
-pub struct WasmCtx {
-    id: CtxId,
+/// Default WasmContext implementation.
+#[derive(Debug, Clone)]
+pub struct WasmCtx<C: Message = Empty> {
+    pub(crate) size: usize,
+    pub(crate) _priv: PhantomData<C>,
 }
 
-impl WasmCtx {
-    pub(crate) fn id(&self) -> CtxId {
-        self.id
-    }
-    pub(crate) fn from_id(id: CtxId) -> WasmCtx {
-        WasmCtx { id }
-    }
-}
-
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct CodeMsg {
     pub code: i32,
     pub msg: String,
@@ -42,36 +36,17 @@ impl CodeMsg {
     pub fn result<T, S: ToString>(code: i32, msg: S) -> Result<T> {
         Err(Self::new(code, msg))
     }
-}
-
-pub const ERR_CODE_UNKNOWN: ErrCode = ErrCode(-1);
-pub const ERR_CODE_PROTO: ErrCode = ErrCode(-2);
-pub const ERR_CODE_NONE: ErrCode = ErrCode(-3);
-pub const ERR_CODE_MEM: ErrCode = ErrCode(-4);
-
-pub struct ErrCode(i32);
-
-impl ErrCode {
-    pub fn value(&self) -> i32 {
-        self.0
-    }
-    #[inline]
-    pub fn to_code_msg<S: ToString>(&self, msg: S) -> CodeMsg {
-        CodeMsg::new(self.0, msg)
-    }
-    #[inline]
-    pub fn to_result<T, S: ToString>(&self, msg: S) -> Result<T> {
-        CodeMsg::result(self.0, msg)
+    pub fn into_result<T>(self) -> Result<T> {
+        Err(self)
     }
 }
 
-impl Deref for ErrCode {
-    type Target = i32;
+pub type RetCode = i32;
 
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
+pub const CODE_UNKNOWN: RetCode = -1;
+pub const CODE_PROTO: RetCode = -2;
+pub const CODE_NONE: RetCode = -3;
+pub const CODE_MEM: RetCode = -4;
 
 impl std::fmt::Display for CodeMsg {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
@@ -84,7 +59,7 @@ impl From<anyhow::Error> for CodeMsg {
         if e.is::<CodeMsg>() {
             e.downcast().unwrap()
         } else {
-            ERR_CODE_UNKNOWN.to_code_msg(e)
+            CodeMsg::new(CODE_UNKNOWN, e)
         }
     }
 }
@@ -97,13 +72,13 @@ impl From<CodeMsg> for anyhow::Error {
 
 impl From<std::io::Error> for CodeMsg {
     fn from(e: std::io::Error) -> Self {
-        ERR_CODE_UNKNOWN.to_code_msg(format!("io: {}", e))
+        CodeMsg::new(CODE_UNKNOWN, format!("io: {}", e))
     }
 }
 
 impl From<protobuf::ProtobufError> for CodeMsg {
     fn from(e: protobuf::ProtobufError) -> Self {
-        ERR_CODE_PROTO.to_code_msg(format!("protobuf: {}", e))
+        CodeMsg::new(CODE_PROTO, format!("protobuf: {}", e))
     }
 }
 
@@ -113,9 +88,9 @@ impl<R: Message> From<OutRets> for Result<R> {
             return CodeMsg::result(out_rets.get_code(), out_rets.get_msg());
         }
         out_rets.get_data()
-                  .unpack::<R>()?
+                .unpack::<R>()?
             .map_or_else(
-                || ERR_CODE_PROTO.to_result("protobuf: the message type does not match the out_rets"),
+                || CodeMsg::result(CODE_PROTO, "protobuf: the message type does not match the out_rets"),
                 |data| Ok(data),
             )
     }
@@ -132,7 +107,7 @@ impl InArgs {
         self.get_data()
             .unpack::<R>()?
             .map_or_else(
-                || ERR_CODE_PROTO.to_result("protobuf: the message type does not match the in_args"),
+                || CodeMsg::result(CODE_PROTO, "protobuf: the message type does not match the in_args"),
                 |data| Ok(data),
             )
     }
@@ -141,7 +116,7 @@ impl InArgs {
 pub fn unpack_any<R: Message>(data: &Any) -> Result<R> {
     data.unpack::<R>()?
         .map_or_else(
-            || ERR_CODE_PROTO.to_result("protobuf: the message type does not match the data"),
+            || CodeMsg::result(CODE_PROTO, "protobuf: the message type does not match the data"),
             |r| Ok(r),
         )
 }
@@ -185,7 +160,7 @@ impl<R: Message> From<Result<R>> for OutRets {
                         res.set_data(data);
                     }
                     Err(err) => {
-                        res.set_code(ERR_CODE_PROTO.0);
+                        res.set_code(CODE_PROTO);
                         res.set_msg(err.to_string());
                     }
                 }
@@ -198,7 +173,15 @@ impl<R: Message> From<Result<R>> for OutRets {
 
 impl FromResidual<Option<Infallible>> for OutRets {
     fn from_residual(_residual: Option<Infallible>) -> Self {
-        ERR_CODE_NONE.to_code_msg("not found").into()
+        CodeMsg::new(CODE_NONE, "not found").into()
     }
 }
 
+impl FromResidual<Result<Infallible>> for OutRets {
+    fn from_residual(residual: Result<Infallible>) -> Self {
+        match residual {
+            Err(e) => e.into(),
+            _ => { unreachable!() }
+        }
+    }
+}
